!>
!! Minimum value of a 1D array
!!
Subroutine min_minloc_var(var,blocking,minofvar,mloc)
  Use precision
  Use constants
!$ Use omp_lib

  Use global_variables, ONLY: nijk, kblk, numthreads

  Real(iprec), Dimension(nijk), Intent(in):: var
  Logical, Intent(in):: blocking
  Real(iprec), Intent(out):: minofvar
  Integer, Dimension(3), Optional, Intent(out):: mloc

  Integer:: ii, iimin, i, j, k, ierr
  Real(iprec):: varii, block
  !-
  !- calculate location of minimum
  !-
  Integer:: ithread
  Integer, Dimension(:), Allocatable:: iimnth
  Real(iprec), Dimension(:), Allocatable:: minth
  Real(iprec), Dimension(1):: mnloc

  block = zero
  if (blocking) block = one

  iimin = 0
  minofvar = infty

  Allocate(minth(numthreads),iimnth(numthreads),STAT=ierr)
  if (ierr /= 0) call allocationerror("min_minloc_var:minth")
  minth(:)  = infty
  iimnth(:) = 0

  !$OMP Parallel Do Default(shared) Private(ii,ithread,varii)
  Do ii = 1,nijk

    ithread = 1
!$  ithread = omp_get_thread_num() + 1

    varii = (one-block)*var(ii) &
                +block *(var(ii)*kblk(ii) + (1 - kblk(ii))*infty)

    if (varii < minth(ithread)) then
      minth (ithread) = varii
      iimnth(ithread) = ii
    endif
  Enddo
  !$OMP End Parallel Do

  minofvar = MINVAL(minth)
  mnloc    = MINLOC(minth)
  iimin    = iimnth(mnloc(1))

  If (PRESENT(mloc)) then
    call ind1t3(iimin,i,j,k)
    mloc(1) = i
    mloc(2) = j
    mloc(3) = k
  Endif
End Subroutine min_minloc_var

!------------------------------!
! Maximum value of a 1D array -!
!------------------------------!
Subroutine max_maxloc_var(var,blocking,maxofvar,mloc)
  Use precision
  Use constants
!$ Use omp_lib

  Use global_variables, ONLY: nijk, kblk, numthreads

  Real(iprec), Dimension(nijk), Intent(in):: var
  Logical, Intent(in):: blocking
  Real(iprec), Intent(out):: maxofvar
  Integer, Dimension(3), Optional, Intent(out):: mloc

  Integer:: ii, iimax, i, j, k, ierr
  Real(iprec):: varii, block
  !-
  !- calculate location of maximum
  !-
  Integer:: ithread
  Integer, Dimension(:), Allocatable:: iimxth
  Real(iprec), Dimension(:), Allocatable:: maxth
  Real(iprec), Dimension(1):: mxloc

  block = zero
  if (blocking) block = one

  iimax = 0
  maxofvar = -infty

  Allocate(maxth(numthreads),iimxth(numthreads),STAT=ierr)
  if (ierr /= 0) call allocationerror("max_maxloc_var:maxth")
  maxth(:)  = -infty
  iimxth(:) = 0

  !$OMP Parallel Do Default(shared) Private(ii,ithread,varii)
  Do ii = 1,nijk

    ithread = 1
!$  ithread = omp_get_thread_num() + 1

    varii = (one-block)*var(ii) &
                +block *(var(ii)*kblk(ii) - (1 - kblk(ii))*infty)

    if (varii > maxth(ithread)) then
      maxth (ithread) = varii
      iimxth(ithread) = ii
    endif
  Enddo
  !$OMP End Parallel Do

  maxofvar = MAXVAL(maxth)
  mxloc    = MAXLOC(maxth)
  iimax    = iimxth(mxloc(1))

  If (PRESENT(mloc)) then
    call ind1t3(iimax,i,j,k)
    mloc(1) = i
    mloc(2) = j
    mloc(3) = k
  Endif
End Subroutine max_maxloc_var
